AceAutoInvite		= AceAddonClass:new({
	name			= "AceAutoInvite",
	description		= AAI_DESC,
	version			= "4",
	aceCompatible	= "103",
	author			= "Silverwind",
	category		= "raid",
	db				= AceDatabase:new("AceAutoInviteDB"),
	cmd				= AceChatCmdClass:new({"/aai"},
		{
			{
				option = "keyword", desc = AAI_SET_KEYWORD, method  = "SetKeyWord", input = TRUE
			},
			{
				option = "raid", desc = AAI_RAID_FORMING, method = "ToggleRaid"
			},
			{
				option = "autoaccept", desc = AAI_TOGGLE_AA, method = "ToggleAA"
			},
			{
				option = "autopromote", desc = AAI_TOGGLE_AP, method = "ToggleAP"
			},
			{
				option = "sendwhispers", desc = AAI_TOGGLE_SW, method = "ToggleSW"
			},
			{
				option = "guildonly", desc = AAI_TOGGLE_GUILD, method = "ToggleGuild"
			},
		}
	)
})

local KeyWord = "invite"
local RaidEnabled = "on"
local AutoAccept = "off"
local AutoPromote = "off"
local SendWhispers = "on"
local GuildOnly = "off"

function AceAutoInvite:Enable()
	if not self:Get("KeyWord") then self:Set("KeyWord",KeyWord) else KeyWord = self:Get("KeyWord") end
	if not self:Get("RaidEnabled") then self:Set("RaidEnabled",RaidEnabled) else RaidEnabled = self:Get("RaidEnabled") end
	if not self:Get("AutoAccept") then self:Set("AutoAccept",AutoAccept) else AutoAccept = self:Get("AutoAccept") end
	if not self:Get("AutoPromote") then self:Set("AutoPromote",AutoPromote) else AutoPromote = self:Get("AutoPromote") end
	if not self:Get("SendWhispers") then self:Set("SendWhispers",SendWhispers) else SendWhispers = self:Get("SendWhispers") end
	if not self:Get("GuildOnly") then self:Set("GuildOnly",GuildOnly) else GuildOnly = self:Get("GuildOnly") end
	self:RegisterEvent("PLAYER_ENTERING_WORLD","HandleEvents")
end

function AceAutoInvite:HandleEvents()
	if AutoAccept == "on" then self:RegisterEvent("PARTY_INVITE_REQUEST") else self:UnregisterEvent("PARTY_INVITE_REQUEST") end
	if AutoPromote == "on" or SendWhispers == "on" then self:RegisterEvent("CHAT_MSG_SYSTEM") else self:UnregisterEvent("CHAT_MSG_SYSTEM") end
	self:RegisterEvent("CHAT_MSG_WHISPER")
end

function AceAutoInvite:SetKeyWord(msg)
	if msg then
		KeyWord = msg
		self:Set("KeyWord",KeyWord)
		self:Msg(AAI_NEW_KEYWORD..msg)
	else
		self:Msg(AAI_SPECIFY_KEYWORD)
	end
end

function AceAutoInvite:ToggleRaid()
	if RaidEnabled == "on" then
		RaidEnabled = "off"
		self:Set("RaidEnabled",RaidEnabled)
		self:Msg(AAI_RAID_OFF)
	else
		RaidEnabled = "on"
		self:Set("RaidEnabled",RaidEnabled)
		self:Msg(AAI_RAID_ON)
	end
end

function AceAutoInvite:ToggleGuild()
	if GuildOnly == "on" then
		GuildOnly = "off"
		self:Set("GuildOnly",GuildOnly)
		self:Msg(AAI_EVERYONE)
	else
		GuildOnly = "on"
		self:Set("GuildOnly",GuildOnly)
		self:Msg(AAI_GUILDONLY)
	end
end

function AceAutoInvite:ToggleAA()
	if AutoAccept == "on" then
		AutoAccept = "off"
		self:Set("AutoAccept",AutoAccept)
		self:Msg(AAI_AA_OFF)
	else
		AutoAccept = "on"
		self:Set("AutoAccept",AutoAccept)
		self:Msg(AAI_AA_ON)
	end
	self:HandleEvents()
end

function AceAutoInvite:ToggleAP()
	if AutoPromote == "on" then
		AutoPromote = "off"
		self:Set("AutoPromote",AutoPromote)
		self:Msg(AAI_AP_OFF)
	else
		AutoPromote = "on"
		self:Set("AutoPromote",AutoPromote)
		self:Msg(AAI_AP_ON)
	end
	self:HandleEvents()
end

function AceAutoInvite:ToggleSW()
	if SendWhispers == "on" then
		SendWhispers = "off"
		self:Set("SendWhispers",SendWhispers)
		self:Msg(AAI_SW_OFF)
	else
		SendWhispers = "on"
		self:Set("SendWhispers",SendWhispers)
		self:Msg(AAI_SW_ON)
	end
	self:HandleEvents()
end

function AceAutoInvite:CHAT_MSG_WHISPER()
	if string.find(string.lower(arg1),string.lower(KeyWord)) then self:Invite(arg2) end
end

function AceAutoInvite:CHAT_MSG_SYSTEM()
	local name
	if AutoPromote == "on" then
		_, _, name = string.find(arg1,"(.+) has joined the raid group.")
		if name and IsRaidLeader() then PromoteToAssistant(name) end
	end
	if SendWhispers == "on" then
		_, _, name = string.find(arg1,"(.+) is already in a group.")
		if name then SendChatMessage(AAI_WHISPER,"WHISPER",nil,name) end
	end
end

function AceAutoInvite:PARTY_INVITE_REQUEST()
	if AutoAccept == "on" then
		AcceptGroup()
		StaticPopup_Hide("PARTY_INVITE")
	end
end

function AceAutoInvite:SendInvite(playerToInvite)
	if GuildOnly == "on" then
		if IsPlayerInGuild(playerToInvite) then InviteByName(playerToInvite) end
	else
		InviteByName(playerToInvite)
	end
end

function AceAutoInvite:Invite(playerName)
	local numGroup = GetNumPartyMembers()
	local numRaid = GetNumRaidMembers()
	if numRaid == 0 then
		if numGroup == 0 then 
			self:SendInvite(playerName)
		elseif numGroup < 4 then
			if not IsPartyLeader() then self:Msg(AAI_NOT_GROUP_LEADER) return end
			self:SendInvite(playerName)
		elseif numGroup == 4 then
			if not IsPartyLeader() then self:Msg(AAI_NOT_GROUP_LEADER) return end
			if RaidEnabled == "on" then
				ConvertToRaid()
				self:SendInvite(playerName)
			end
		end
	elseif numRaid == 40 then 
		self:Msg(AAI_RAID_FULL)
	else 
		if IsRaidLeader() or IsRaidOfficer() then 
			self:SendInvite(playerName)
		else
			self:Msg(AAI_NOT_PROMOTED_OR_LEADER)
		end
	end
end

function AceAutoInvite:IsPlayerInGuild(playerName)
	local i
	for i=1, GetNumGuildMembers() do
		local name, _, _, _, _, _, _, _, _, _ = GetGuildRosterInfo(i)
		if playerName == name then return true end
	end
	return false
end
AceAutoInvite:RegisterForLoad()

ace:RegisterFunctions(AceAutoInvite, 
	{
		Msg  = function(self, ...) self.cmd:msg(unpack(arg)) end,
		Get  = function(self, var) return self.db:get(self.profilePath, var) end,
		Set  = function(self, var, val) self.db:set(self.profilePath, var, val) end,
	}
)
